/*
  In App.xaml:
  <Application.Resources>
      <vm:ViewModelLocator xmlns:vm="clr-namespace:SHM.UI"
                           x:Key="Locator" />
  </Application.Resources>
  
  In the View:
  DataContext="{Binding Source={StaticResource Locator}, Path=ViewModelName}"

  You can also use Blend to do all this with the tool's support.
  See http://www.galasoft.ch/mvvm
*/

using CommonServiceLocator;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Ioc;
using MahApps.Metro.Controls.Dialogs;
using SHM.UI.Extensions;
using SHM.Utilities;

namespace SHM.UI.ViewModel
{
    /// <summary>
    /// This class contains static references to all the view models in the
    /// application and provides an entry point for the bindings.
    /// </summary>
    public class ViewModelLocator
    {
        /// <summary>
        /// Initializes a new instance of the ViewModelLocator class.
        /// </summary>
        public ViewModelLocator()
        {
            ServiceLocator.SetLocatorProvider(() => SimpleIoc.Default);

            ////if (ViewModelBase.IsInDesignModeStatic)
            ////{
            ////    // Create design time view services and models
            ////    SimpleIoc.Default.Register<IDataService, DesignDataService>();
            ////}
            ////else
            ////{
            ////    // Create run time view services and models
            ////    SimpleIoc.Default.Register<IDataService, DataService>();
            ////}
            SimpleIoc.Default.Reregister<ViewModelLocator>();
            SimpleIoc.Default.Register<MainViewModel>();
            SimpleIoc.Default.Register<HomeViewModel>();
            SimpleIoc.Default.Register<HomebrewsViewModel>();
            SimpleIoc.Default.Register<DetailsViewModel>();
            SimpleIoc.Default.Register<SettingsViewModel>();
            SimpleIoc.Default.Register<UpdateViewModel>();
        }

        TModel Get<TModel>() => ServiceLocator.Current.GetInstance<TModel>();
        public MainViewModel Main => Get<MainViewModel>();
        public HomeViewModel Home => Get<HomeViewModel>();
        public HomebrewsViewModel Homebrews => Get<HomebrewsViewModel>();
        public DetailsViewModel Details => Get<DetailsViewModel>();
        public SettingsViewModel Settings => Get<SettingsViewModel>();
        public UpdateViewModel Update => Get<UpdateViewModel>();

        public static void Cleanup()
        {
            // TODO Clear the ViewModels
        }
    }
}